#!/usr/bin/perl

#
# weblogUpdatesPing
# version: 0.1a
# date:    2004.08.31
# author:  Joerg Feuerhake
# website: http://www.tageloehner.de/feuerhake/
#
# Copyright (C) 2004 Joerg Feuerhake.
#
# Specification: "http://www.xmlrpc.com/weblogsCom".
# Directory: "http://www.weblogs.com/"
#
# This script is based on Hans Kellners weblogUpdatesPing.pl written in 2001.
# However, it did not fit my needs to ping several services at once.
# Furthermore I refactored the code a little, just to learn how the whole
# xmlrpc thing works. I hope you will get along with it. So, this is
# the enhanced version of Hans Kellners (http://www.hanskellner.com/) script.

#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

use IO::Socket qw(:DEFAULT :crlf);

# PLEASE GIVE the name, url and rss of your Weblog.
# For example:
#    $weblog_name = "Go Mountain Biking!";
#    $weblog_url  = "http://www.gomtb.com/";
#    $weblog_rss  = "http://www.gomtb.com/rss.xml";

$weblog_name = "";
$weblog_url  = "";
$weblog_rss  = "";

# PLEASE SET THE THREE VALUES RIGHT ABOVE THIS COMMENT BLOCK
#
# this method pings weblogs, technorati, blogg.de
# it takes the following parameters:
# $name: the name of the weblog
# $url: the url of the weblog
# $rss: the rss of the weblog
#
# PLEASE SET THE THREE VALUES RIGHT ABOVE THIS COMMENT BLOCK
#
sub weblogUpdatesPing($$$){

    my $name = shift;
    my $url  = shift;
    my $rss  = shift;

    my $pingRequest  = "";
    my $returnString = "(-1): WeblogUpdatePing Unknown Error";
    my $RequestStringError = "(-1): MethodName, BlogName or BlogUrl does not seem to be set properly";

    #first rpc.weblog.com
      $pingRequest = setPingString("false", "weblogUpdates.ping", $name, $url, $rss);
      if(!$pingRequest){
      	return $RequestStringError; };
      $returnString = "weblogs.com".pingWeblogServer("rpc.weblogs.com", "/RPC2", $pingRequest);

    #next rpc.technorati.com
      $pingRequest = setPingString("false", "weblogUpdates.ping", $name, $url, $rss);
      if(!$pingRequest){
      	return $RequestStringError; };
      $returnString = $returnString."\n\n technorati".
      		pingWeblogServer("rpc.technorati.com", "/RPC2", $pingRequest);

    #last one xmlrpc.blogg.de (this is the extrawurst service, notice changing parameters)
      $pingRequest = setPingString("true", "weblogUpdates.ping", $name, $url, $rss);
      if(!$pingRequest){
      	return $RequestStringError; };
      $returnString = $returnString."\n\n blogg.de".
      		pingWeblogServer("xmlrpc.blogg.de", "/", $pingRequest);

      return $returnString;

}

# Evaluates http responses. this method is called by 'pingWeblogServer'
# It does not need to be called from other methods
# takes as parameter:
#	thisResponse: a serverresponse
#returns: a string of form '($retCode):$retDescr'
sub evalResponse($){

my $thisResponse = shift;
my $ret = "null";

	# The response is in XML.  Retrieve the return code and message.
	$retCode = $thisResponse;
	$retMsg  = $thisResponse;

	# Strip out the XML up to and after the flerror code.
	$retCode =~ s/.*<boolean>//si;
	$retCode =~ s/<\/boolean>.*//si;

	# Strip out the XML up to and after the message.
	$retMsg =~ s/.*<\/boolean>//si;
	$retMsg =~ s/.*<value>//si;
	$retMsg =~ s/<\/value>.*//si;

	# Is there an additional string tag?
	if($retMsg =~ /<string>/){
		$retMsg =~ s/.*<string>//si;
        	$retMsg =~ s/<\/string>.*//si;
	};

        $ret = " ($retCode):$retMsg";
	return $ret;

}

# Pings the specified server and returns flerror and the flerror-description
# takes as arguments:
#  	server: "www.weblogs.com"
#	serverpath: "/RPC2" (this is important, cause the serverpathes differ for some services)
#	pingString: should be the result of our 'setPingString' method
#
sub pingWeblogServer($$$){

my $server = shift;
my $serverpath = shift;
my $pingStr = shift;

 # Connect to the weblogs server
    my $remote = IO::Socket::INET->new(
        Proto    => "tcp",
        PeerAddr => "$server",
        PeerPort => "http(80)" );

    if ( $remote )
    {
        $remote->autoflush(1); # No buffering

        # Send the ping
        print $remote "POST $serverpath HTTP/1.0$CRLF";
        print $remote "User-Agent: tageloehner.de/feuerhake/ weblogUpdatesPing tool$CRLF";
        print $remote "Host: $server$CRLF";
        print $remote "Content-Type: text/xml$CRLF";
        print $remote "Content-length: " . length($pingStr) . "$CRLF$CRLF";
        print $remote $pingStr;

        # Read the response
        $sockResp = "";
        while ( <$remote> ) {
            $sockResp = $sockResp . $_;
        };

	#using our little eval method
	$ret = evalResponse($sockResp);

	# Close the socket
        close $remote;
 	}
    else
    {
        $ret = "(-1) : Cannot connect to http daemon";
    }
    return $ret;
}

# This method sets the request to ping the rpc service
# Since methodcalls differ between services it takes quite a number
# of arguments:
# $str: set this "true" and 'string' tags will be written (blogg.de requires this)
# $method: name of the method to call at the remote server. should be set to "weblogUpdates.ping"
# $name: the name of your blog
# $url: url of your weblog
# $rss: the newsfeed of your blog
#
sub setPingString($$$$$){

    my $str = shift;
    my $method = shift;
    my $name = shift;
    my $url  = shift;
    my $rss  = shift;

    my $strtag = "";
    my $strendtag = "";

    if($str =~ "true"){
	$strtag = "<string>";
	$strendtag = "</string>";
    };

    if ( !$name || !$url || !$method) {
        return;
    };

    # This is the xml-rpc request
    my $pingStr =
        "<?xml version=\"1.0\"?>" .
        "<methodCall>" .
        "    <methodName>$method</methodName>" .
        "    <params>" .
        "        <param><value>$strtag$name$strendtag</value></param>" .
        "        <param><value>$strtag$url$strendtag</value></param>" ;
    if ( $rss && $rss !~ ""){
	$pingStr = $pingStr."        <param><value>$strtag$rss$strendtag</value></param>" ;
	};
        $pingStr = $pingStr."    </params>" .
            "</methodCall>";

	    return $pingStr;
}

# Now, check whether everything works properly,
# Ping the server. Good luck!
print "\nPinging rpc.weblogs.com, technorati.com,".
	" blogg.de for weblog \"$weblog_name\" at \"$weblog_url\"\n\n**\n";

#call the main method here
$response = weblogUpdatesPing( $weblog_name, $weblog_url, $weblog_rss );

#print the result
print "$response\n\n**\n";

